#ifndef __BURDEN_H__
#define __BURDEN_H__

#include "FastFit.h"
#include "VcfRecord.h"
#include "VcfFileReader.h"
#include "VcfHeader.h"
#include "SingleVariantTest.h"

class Burden
{
    public:
        Burden();
        ~Burden();
        static bool vcfAnnotated;
        static bool burden;
        static bool LinScoreAssoc;
        static String genename;
        static String genename_f;
        static double mafMin;
        static double mafMax;
        static String function;
        static String chromosome;
	
	static bool CMC_binary;
	static bool CMC_counts;
	static bool MB;
	static bool SCORE;
	static bool LRT;
	static bool WALD;
	static bool SKAT;

	//Input/Output options	
        static String vcfInput;
        static String outputFile;


	//this is the total number of genes
	int geneCount,SNPcount;
	double mac,cum_maf;
	int numVarPerGene;
	double markerMaf;
	Vector genotypeMB, genotypeCMC_b, genotypeCMC_c;
	Vector mafArray;
	Matrix genotypeSKAT,P0;
	Vector residuals_original,fixedEff,beta_hat,weight;
	double sigma2_hat;

	VcfFileReader reader;
	VcfHeader header;
	VcfRecord record;

	void GetAllGenenamesVCF(String & chromName);
	void HashInGenes(std::string & gene);

	bool GetGenotypeVector(FastTransform & trans, Pedigree & ped,String & gene,String & chrom, int start,int end,SVT & svt);
	void Calculate(Pedigree & ped, FastTransform & trans,FastFit & engine,String & traitName,SVT & svt);
	void Run(Pedigree & ped, FastTransform & trans, FastFit & engine,String & traitName,SVT & svt);	
	void LinAssoc(FastTransform & trans,FastFit & engine, FILE * out, Vector & genotype,String & geneName,String & method);
	void MerlinFastAssoc(FastTransform & trans,FILE * out, Vector & genotype,String & geneName,FastFit & engine,String & method);
	void MerlinLRT(double L0,FastTransform & fullTrans,FILE * out, Vector & genotype,String & geneName,FastFit & fullEngine,String & method);
	void WaldAssoc(FastTransform & fullTrans,FILE * out, Vector & genotype,String & geneName,FastFit & fullEngine,String & method);
	void SKATScoreAssoc(FastTransform & trans,FILE * out,String & geneName,FastFit & engine);
	void PrintHeader(FILE * output,FastTransform & trans,FastFit & engine,String & traitName);
	void PrintSKATHeader(FILE * output,FastTransform & trans,FastFit & engine,String & traitName);
	void PrintFooter(FILE * output,int bad);
	void CalculateLambda(Vector & weight,double lambda [],Matrix & transGeno);
	double MixChidist(double lambda [], int n, double q_obs,String method);
	double GetBetaDensity(double a, double b, double x);
};

#endif
