#ifndef __BURDEN_H__
#define __BURDEN_H__

#include "VcfRecord.h"
#include "VcfFileReader.h"
#include "VcfHeader.h"

bool SVT::GetGenotypeVectorVCF(FastTransform & trans, Pedigree & ped,int markerIdx)
{
   bool status = true;
   genotype.Dimension(trans.persons);

   reader.open(Burden::vcfInput,header);
   int numSamples = header.getNumSamples();

   unsigned int recordCount =0;
   int n=0,nf=0;
   double fmaf = 0.0, maf=0.0; 
   mac=0.0;

   while(reader.readRecord(record))
   {
      if(markerIdx==recordCount)
      {
	 rareAllele = record.getAltStr();
	 chr = record.getChromStr();
	 pos = record.get1BasedPosition();
	 VcfRecordGenotype & genoInfo = record.getGenotypeInfo();
	 //Loop througth all samples to calculate maf
	 std::vector<int> fcounts;
	 std::vector<int> counts;

	 for(int i = 0; i < numSamples; i++)
	 {
	    int numGTs = record.getNumGTs(i);
	    Vector allele;
	    for(int j = 0; j < numGTs; j++)
	    {
	       allele.Push(getGT(i,j));
	    }
	    int bad =0;
	    for(int m=0;m<allele.Length();m++)
	    {
	       bad += (allle[m]==VcfGenotypeSample::MISSING_GT);
	    }
	    if(bad>0)
	    {
	       //skip this sample
	       break;
	    }

	    //judge if this sampel is a founder
	    bool founder = false;
	    for(int f=0;f<trans.founders.Length();f++)
	    {
	       if(strcmp(trans.founders[f].c_str(),header.getSampleName(f))==0)
	       {
		  founder=true;
		  break;
	       }
	    }

	    for(int m=0;m<allele.Length();m++)
	    {
	       if(allele[m] >= counts.size())
	       {
		  counts.resize(allele[m]+1);
	       }
	       ++counts[allele[m]];
	       if(founder)
	       {
		  if(allele[m] >= fcounts.size())
		  {
		     fcounts.resize(allele[m]+1);
		  }
		  ++fcounts[allele[m]];
	       }
	    }
	    //Only consider diallelic markers
	    if(counts.size()>2)
	    {
	       status = false;
	       founderMaf = 0.0;
	       markerMaf = 0.0;
	       return status;
	    }
	 }
	 //if mornomorphic, then skip this marker
	 if(counts.size()==1 || fcounts.size()==1)
	 {
	    status = false;
	    founderMaf = 0.0;
	    markerMaf = 0.0;
	    return status;
	 }

	 //Calculate maf
	 double sum = 0.0;
	 for(int i=0;i<counts.size();i++)
	 {
	    sum+= counts[i];
	 }
	 markerMaf = counts[1]/sum;
	 double sum = 0.0;
	 for(int i=0;i<fcounts.size();i++)
	 {
	    sum+= fcounts[i];
	 }
	 founderMaf = fcounts[1]/sum;

	 //Initializing genotype vector
	 if(SVT::FounderFreq)
	 {
	    for(int i=0;i<trans.persons;i++)
	       genotype[i] = 2.0*founderMaf;
	 }
	 else
	 {
	    for(int i=0;i<trans.persons;i++)
	       genotype[i] = 2.0*markerMaf;
	 }

	 //Loop through samples according to trans.pPheno 
	 //read in dosage into genotype vector
	 int idx=0;
	 for (int f = 0; f < ped.familyCount; f++)
	 {
	    for(int j=0;j<trans.pPheno[f].Length();j++)
	    {
	       char * sampleID = ped[trans.pPheno[f][j]].pid.c_str();
	       for(int s =0; s<numSamples;s++)
	       {
		  if(strcmp(sampleID,VcfHeader::getSampleName(s)) == 0)
		  {
		     int numGTs = record.getNumGTs(s);
		     Vector allele;

		     for(int j = 0; j < numGTs; j++)
		     {
			allele.Push(getGT(s,j));
		     }
		     int bad =0;
		     for(int m=0;m<allele.Length();m++)
		     {
			bad += (allle[m]==VcfGenotypeSample::MISSING_GT);
		     }
		     //if any allele is missing then use 2*maf to impute
		     if(bad>0)
		     {
			if(FounderFreq)
			   genotype[idx] = 2*founderMaf;
			else 
			   genotype[idx] = 2*markerMaf;
		     }
		     else 
		     {
			int sum=0;
			for(int i=0;i<allele.size();i++)
			   sum+=allele[i];
			genotype[idx] = sum;	
			mac += genotype[idx];
		     }
		     break;
		  }
	       }
	       idx++;
	    }
	 }
	 //Adjusting genotype vector by 2*maf
	 if(SVT::FounderFreq)
	 {
	    for(int i=0;i<trans.persons;i++)
	       genotype[i] -= 2.0*founderMaf;
	 }
	 else
	 {
	    for(int i=0;i<trans.persons;i++)
	       genotype[i] -= 2.0*markerMaf;
	 }
	 break;
      }
      recordCount++;
   }
   return status;
}

bool GetGenotypeVector(FastTransform & trans, Pedigree & ped,int markerIdx)
{
   //get the founders into a string array founders;
   StringArray founders;
   for (int f = 0; f < ped.familyCount; f++)
   {
      for(int j=0;j<trans.pPheno[f].Length();j++)
      {
	 if(ped[trans.pPheno[f][j]].isFounder())
	    founders.Push(ped[trans.pPheno[f][j]].pid);
      }
   }
   bool status;
   if(dosage)
   {
      status = GetGenotypeVectorVCFDosage(trans,ped,markerIdx,founders);
   }
   if(genoFromPed)
   {
      status = GetGenotypeVectorPED(trans,ped,markerIdx);
   }
   if(genoFromVCF)
   {
      status = GetGenotypeVectorVCF(trans,ped,markerIdx,founders);
   }
   return status;
}

bool SVT::GetGenotypeVectorSingleVarVCFDosage(FastTransform & trans, Pedigree & ped,int markerIdx)
{
   //Write the subset samples to keep into subsetFile1.txt
   //delimited by \n
   /*   
	FILE * out;
	out = fopen("subsetFile1.txt","wt");
	for (int f = 0; f < ped.familyCount; f++)
	{
	for(int j=0;j<trans.pPheno[f].Length();j++)
	{
	fprintf(out,"%s\n",ped[trans.pPheno[f][j]].pid.c_str());
	}
	}  
	fclose(out); 
   //Only read in the selected samples
   reader.open(Burden::vcfInput, header, "subsetFile1.txt", NULL, NULL);
    */

   bool status = false;
   genotype.Dimension(trans.persons);

   reader.open(Burden::vcfInput,header);
   int numSamples = header.getNumSamples();
   unsigned int recordCount =0;
   while(reader.readRecord(record))
   {
      if(markerIdx==recordCount)
      {
	 VcfRecordGenotype & genoInfo = record.getGenotypeInfo();
	 int n=0,nf=0;
	 float fmaf = 0.0,maf=0.0;

	 const std::string * geno = getInfo.getString(dosage,s);
	 if(geno==NULL)
	 {
	    error("Error: Can not find DS field for sample %d ...",s);
	 }
	 if(*geno == ".")
	 {
	    continue;
	 }
	 float dose = atof(*geno.c_str());

	 //Calculate founder based maf
	 if(isFounder())
	 {
	    fmaf += dose;
	    nf++;
	 }

	 //Calculate overall maf
	 maf += dose;
	 n++;
      }

      //Initializing genotype vector
      if(SVT::FounderFreq)
      {
	 for(int i=0;i<trans.persons;i++)
	    genotype[i] = 2.0*fmaf;
      }
      else
      {
	 for(int i=0;i<trans.persons;i++)
	    genotype[i] = 2.0*maf;
      }

      //Loop through samples according to trans.pPheno 
      //read in dosage into genotype vector
      int idx=0;
      for (int f = 0; f < ped.familyCount; f++)
      {
	 for(int j=0;j<trans.pPheno[f].Length();j++)
	 {
	    char * sampleID = ped[trans.pPheno[f][j]].pid.c_str();
	    for(int s =0; s<numSamples;s++)
	    {
	       if(strcmp(sampleID,VcfHeader::getSampleName(s)) == 0)
	       {
		  const std::string * geno = getInfo.getString(dosage,s);
		  if(strcmp(*geno,".")>0)
		  {
		     float dose = atof(*geno.c_str());
		     genotype[idx] = dose;
		  }
		  break; 
	       }
	    }
	    idx++;
	 }
      }  
      //Adjusting genotype vector by 2*maf
      if(SVT::FounderFreq)
      {
	 for(int i=0;i<trans.persons;i++)
	    genotype[i] -= 2.0*fmaf;
      }
      else 
      {
	 for(int i=0;i<trans.persons;i++)
	    genotype[i] -= 2.0*maf;
      }
      break;
   }
   recordCount++;
}


































#endif
